import os, sys, logging
from ConfigParser import ConfigParser

"""
In this file we assume we have none of the logging facilities or anything that
the majority of Thump has available.  If something goes wrong, just print to
stderr and die.
"""

class Configuration(ConfigParser):

  def __init__(self):
    self.homeConfDir = os.path.join(os.path.expanduser("~"), ".thump")

    self._checkDir(self.homeConfDir)

    self.lists = {
      "listDir": os.path.join(self.homeConfDir, "lists"),
      "defaultList": "playlist"
    }

    self._checkDir(self.lists["listDir"])

    basenames = {
     #internal name/actual filename
      "configFile"   : "config",
      "PlayerAccess" : "access_log",
      "PlayerErrors" : "error_log",
      "PlayerPid"    : "player_pid",
      "QueueErrors"  : "q_error_log",
      "QueuePid"     : "q_pid",
      "Stderr"       : "stderr_out"
    }

    self.filenames = dict()
    for ourname, filename in basenames.items():
      self.filenames[ourname] = os.path.join(self.homeConfDir, filename)


    self.logs = {
      "logLevel": "DEBUG",
      "logEntryFormat": "%(asctime)s %(process)s %(levelname)s %(message)s",
      "playerAccessMsg": "Playing file: %s"
    }

    self.addresses = {
      "player": "127.0.0.1",
      "queue": "127.0.0.1"
    }

    # TODO: change so that you can specify to use unix sockets
    # (and edit their filenames here)
    self.ports = {
      #port on which the player listens for msgs from the queue
      "playerToQueue": 4802,
      #port on which the queue listens for msgs from the player
      "queueToPlayer": 4801,
      #port on which the queue listens for input from frontend apps
      "queueToOutside": 4800
    }

    self.toggles = {
      "shuffle": True,
      "repeat": True
    }

    defaults = {}
    map(defaults.update, (self.lists, self.filenames, self.toggles,
                          self.ports, self.addresses, self.logs,
                          { "HOME": os.path.expanduser("~"),
                            "homeConfDir": self.homeConfDir }) )

    ConfigParser.__init__(self, defaults)

    self.update()

  def _checkDir(self, dirname):
    if not os.path.exists(dirname):
      try:
        os.mkdir(dirname, 0755)
      except OSError:
        print >> sys.stderr, dirname, \
        "doesn't exist, and I can't create it!\nI need this directory.  HTH, HAND."
        sys.exit(1)

  def update(self):
    self.read(self.filenames["configFile"])


configFile = Configuration()

