try:
  import logging # if python >= 2.3
except ImportError:
  import Thump.logging as logging

from Thump.Configuration import configFile


# base log classes

class Log(object):
  def __new__(cls, name, filename, level, format):
    logger = logging.getLogger(name)

    handler = logging.FileHandler(filename)
    handler.setFormatter(logging.Formatter(format))

    logger.addHandler(handler)

    if isinstance(level, str):
      # make LEVEL as well as "LEVEL" acceptable
      level = logging._levelNames[level]

    logger.setLevel(level)

    return logger


class NullLog:
  critical= fatal= error= warning= warn= info= debug= notset= lambda *a:None


# thump-specific

class ThumpLog(Log):
  def __new__(cls, name, level=configFile.logs["logLevel"], format=configFile.logs["logEntryFormat"]):
    return Log.__new__(cls, 'Thump.%s'%name, configFile.filenames[name], level, format)


class PlayerAccessLog(ThumpLog):
  def __new__(cls):
    log = ThumpLog.__new__(cls, "PlayerAccess")
    log.access = lambda filename: log.info(configFile.logs["playerAccessMsg"] % filename)
    return log


class PlayerErrorLog(ThumpLog):
  def __new__(cls):
    return ThumpLog.__new__(cls, "PlayerErrors")


class QueueErrorLog(ThumpLog):
  def __new__(cls):
    return ThumpLog.__new__(cls, "QueueErrors")

