import socket, xmlrpclib, Queue, threading
from xml.parsers.expat import ExpatError

from SocketServer import StreamRequestHandler, TCPServer, \
UnixStreamServer, ThreadingUnixStreamServer, ThreadingTCPServer

from Thump import Messages


class FamilyError(Exception):
  def __repr__(self):
    return "Family must be one of `tcp' or `unix'"
  __str__ = __repr__


def getFamily(name):
    if name == "tcp":
      return socket.AF_INET
    elif name == "unix":
      return socket.AF_UNIX
    else:
      raise FamilyError


class Client(object):

  def __init__(self, addrFamily, address):
    self.family = getFamily(addrFamily)
    self.address = address

  def send(self, msg):
    if not msg:
      return

    s = socket.socket(self.family, socket.SOCK_STREAM)
    s.connect(self.address)

    print 'Client.send got msg:', msg
    print 'xml:', msg.xml()

    s.send(msg.xml())
    s.close()


class ThreadedQueueMixin(threading.Thread):

  inbox = Queue.Queue()

  def __init__(self):
    if not hasattr(self, "serve_forever"):
      raise Exception, "ThreadedQueueMixin is meant only as a mixin for SocketServers"
    threading.Thread.__init__(self)
    self.setDaemon(True)
  
  def run(self):
    self.serve_forever()


class MyUnixServer(ThreadingUnixStreamServer, ThreadedQueueMixin):
  def __init__(self, addr, handler):
    UnixStreamServer.__init__(self, addr, handler)
    ThreadedQueueMixin.__init__(self)

class MyTcpServer(ThreadingTCPServer, ThreadedQueueMixin):
  def __init__(self, addr, handler):
    while 1:
      try:
        TCPServer.__init__(self, addr, handler)
      except socket.error:
        # XXX log higher up?
        print "couldn't get socket"
        time.sleep(2)
      else:
        print "got socket"
        break

    ThreadedQueueMixin.__init__(self)


class MyHandler(StreamRequestHandler):
  def handle(self):
    line = self.rfile.readline(9192).strip()

    if not line:
      return

    print 'MyHandler: got text:', line

    msg = Messages.Message(line)

    if not msg:
      print "MyHandler: xml was empty or not valid"

    else:
      print 'MyHandler: turned xml into dict:', msg
      self.server.inbox.put(msg)


class SocketCommandCollector(object):
  # The big kahuna

  remoteServer = None

  def __init__(self, myAddr, theirAddr=None):

    self.me = myAddr
    self.them = theirAddr

    # just checking

    getFamily(self.me[0])

    if self.them:
      getFamily(self.them[0])

    # --

    self.setUpServer()
    self.setUpClient()


  def setUpServer(self):
    servtype = (self.me[0] == "tcp" and MyTcpServer or MyUnixServer)

    self.server = servtype(self.me[1], MyHandler)

    self.server.start()

  def setUpClient(self):
    if self.them:
      self.remoteServer = Client(self.them[0], self.them[1])

  def send(self, msg):
    if self.remoteServer:
      self.remoteServer.send(msg)

  def update(self):
    pass

  def size(self):
    return self.server.inbox.qsize()

  def get(self):
    return self.server.inbox.get()





####################  demo  #######################
# yes, this is as ugly as possible :)

if __name__ == "__main__":
  from Thump.Configuration import addresses, ports
  import socket, sys; sys.stderr = sys.stdout
  q = SocketCommandCollector(("tcp", (addresses["queue"], ports["queueToPlayer"])))
  print "listening on port", ports["queueToPlayer"]
  while 1: print q.pop()


