import popen2, os, sys, select, time

import Messages

# The Mp3 class was derived from the mpg123 class, found in the pylayer source.
#
# http://freshmeat.net/projects/pylayer/?topic_id=123
#
# Here's the contents of the LICENSE file from pylayer:
#
# ----------------------------------------------------------------------------
# "THE PARTY-WARE LICENSE" (Revision 37.3):
# This project was developed by Davyd Madeley <davyd@ucc.asn.au> with input from
# others. As long as you retain this notice you can do whatever you want with
# this stuff. If you find p(y)layer worthwile, useful or cool, and hold a party
# with it, you might like to invite us. The authors accept no responsibility for
# usefulness, merchantability or fitness of purpose, and as a result this soft-
# ware comes with no warrenty, implied or otherwise.
# ----------------------------------------------------------------------------


class Mp3(object):

  fields = """
  modeExtension channels bitrate sampleRate  protected  extension
  bytesPerFrame emphasis version copyrighted modeString layer
  """.strip().split()

  frameFields = """
  framesRemaining timeRemaining currentFrame currentTime totalTime
  """.strip().split()

  buf = ""
  done = False
  sentdone = False

  players = [ "mpg321", "mpg123" ]

  opts = ["--skip-printing-frames=20 -R foo 2>/dev/null"] * 2

  assert len(players) == len(opts)

  def __init__(self, filename):
    self.initAttrs()
    self.filename = filename
    self.getDecoder()

  def __del__(self):
    print 'got deleted'
    sys.stdout.flush()
    if not self.done:
      try: self.quit()
      except: pass

  def fileno(self):
    return self.child.fromchild.fileno()

  def size(self):
    if self.done and not self.sentdone:
      return True

  def get(self):
    if self.done and not self.sentdone:
      print 'Mp3: returning stop'
      self.sentdone = True
      return Messages.StopMsg()

  # these 3 are fun

  def initAttrs(self):
    map(lambda pair:map(lambda x:setattr(self,x,pair[0]),pair[1]),
        (("",self.fields),(0,self.frameFields)))

  def updateFields(self, args):
    map(lambda pair:setattr(self,pair[0],pair[1]),
        zip(self.fields, args.split()))

  def updateFrameFields(self, args):
    map(lambda pair:setattr(self,pair[0],pair[1]),
        zip(self.frameFields, args.split()))

  def getDecoder(self):
    for i in range(len(self.players)):
      self.child = popen2.Popen3("%s %s" % (self.players[i], self.opts[i]))
      if self.child.poll() == -1:
        return
 
  def update(self, doSelect=False, timeout=0):
    if doSelect:
      select.select([self],[],[], timeout)

    self.buf += os.read(self.child.fromchild.fileno(), 4096)
    while 1:
      eol = self.buf.find("\n")
      if eol == -1: break

      what, args = self.buf[:eol].split(None,1)

      self.buf = self.buf[eol+1:]

      if what == "@S":
        self.updateFields(args)
      elif what == "@F":
        self.updateFrameFields(args)
        self.totalTime = self.timeRemaining + self.currentTime
      elif (what,args) == ("@P","0"):
        print "mpg321 says it's stopped, running Mp3.quit()"
        self.quit()


  def cmd(self, cmd):
    if self.done:
      return

    self.child.tochild.write("%s\n" % cmd)
    self.child.tochild.flush()

  def play(self):
    # check for 404s on files somewhere
    self.cmd("LOAD %s" % self.filename)
    self.done = self.sentdone = False

  def seek(self, offset):
    self.cmd("JUMP %s" % offset)

  def pause(self):
    self.cmd("PAUSE")

  def quit(self):
    print 'Mp3.quit running'
    self.cmd("QUIT")

    #self.child.wait()

    self.buf = ""
    self.done = True
    print 'Mp3.quit done'


if __name__ == "__main__":
  if len(sys.argv) < 2 or not os.path.isfile(sys.argv[1]):
    print >>sys.stderr, "gimme a file"; sys.exit(1)

  m = Mp3(sys.argv[1])
  m.play()
  try:
    while not m.done: m.update(True)
  except KeyboardInterrupt:
    sys.exit()

